%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ikinescarap.m
% SOLUCIN A LA CINEMTICA INVERSA DEL ROBOT SCARA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Marco Prez, Erik Cuevas, Daniel Zaldvar,  
% Versin: 14/09/2011, v2.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function angulos = ikinescara(t1)
% (1) EVALUA SI EL ARGUMENTO DE ENTRADA ES UNA MATRIZ HT
if ishomog(t1) == 0,
error('Mal Argumento. Debe ser una matriz HT(4x4)')
end

% (2) VERIFICA SEAORIENTACIN REALIZABLE PARA ROBOT SCARA
if (t1(3,3) ~= -1)    
error('Tercera columna no es vector co-linear e inverso al eje Z base')
end

% (3) SE CAPTURA EL MODELO DH DEL ROBOT SCARA
% Se capturan los parmetros de cada eslabn
L1 = link([0 0.25 0 0]);
L2 = link([pi 0.15 0 0]);
L3 = link([0 0 0 0 1]);
L4 = link([0 0 0 0.075]);
% Se enlaza la cadena cinemtica
scara=robot({L1 L2 L3 L4});
% Se definen sus nombres de robot y manufactura
scara.name='SCARA';
scara.manuf='Prototipo';

% (4) SE OBTIENEN LOS VALORES DE LONGITUD A1 Y A2, 
% Y COMPONENTESDE Td: tx, ty,tz, r11 y r12
a1=getfield(L1,'A');
a2=getfield(L2,'A');
d4=getfield(L4,'D');
tx=t1(1,4);
ty=t1(2,4);
tz=t1(3,4);
r11=t1(1,1);
r12=t1(1,2);
% (5) CLCULO DE THETA2, PREVIO AL CLCULO DE M
M=(tx^2+ty^2-(a1^2)-(a2^2))/(2*a1*a2);
ang2=atan2(sqrt(1-M^2), M);
% Note dos aspectos: primero, se toma el valor positivo
% en virtud de la configuracin entre theta1 y theta2.
% Segundo: la funcin atan2 recibe el argumento ty
% en la primera posicin.

% (6) CLCULO DE THETA1
ang1=atan2(ty,tx)-atan2((a2*sin(ang2)),(a1+a2*cos(ang2)) );

% (7) CLCULO DE ESLABN PRISMTICO (desp3)
desp3=-(tz+d4); 
% Se agrega signo negativo para ajustar el valor con
% respecto del sistema coordenado del eslabn prismtico.

% (8) CLCULO DE THETA4, INICIANDO POR PHI
phi=atan2(r12, r11);
ang4=ang1+ang2-phi;

% (9) SE REGRESA EL VECTOR RESULTANTE
angulos=[ang1, ang2, desp3, ang4];
end

