%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% robpendubot.m
% SIMULACIN DEL ROBOT PLANAR DE DOS GRADOS DE LIBERTAD. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Marco Prez, Erik Cuevas, Daniel Zaldivar  
% Versin: 14/09/2011
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% (0)
% Los parmetros de entrada son:
%   velocidad theta1
%   ngulo theta1
%   torque1
%   ngulo theta2
%   velocidad theta2
%
% Esta distribucin permite acomodar las seales
% en el bloque Simulink.
%
% El modelo no considera la friccin.

function y=robpendubot(xin)
% (1)
% Los valores de masa, longitud, distancia al centro de 
% masa e inercias que se utilizan, se han tomado
% del modelo del robot comercial Pendubot.
m1=0.5289;
m2=0.3346;
l1=0.26987;
l2=0.38417;
lc1=0.13494;
lc2=0.19208;
Izz1=0.013863;
Izz2=0.016749;
g=9.81;

% (2) 
% Separacin de los valores de entrada 
dtheta1=xin(1,1);
theta1=xin(2,1);
dtheta2=xin(5,1);
theta2=xin(4,1);
u1=xin(3,1);
% (3)
% Construccin de las matrices del modelo dinmico
% Matriz de Inercia (M)
D11=(m1*(lc1^2))+m2*((l1^2)+(lc2^2)+(2*l1*lc2*cos(theta2)))+Izz1+Izz2;
D12=m2*((lc2^2)+(l1*lc2*cos(theta2)))+Izz2;
D22=(m2*(lc2^2))+Izz2;
D=[D11,D12;D12,D22];

% (4)
% Matriz de Fuerzas Centrfugas y de Coriolis
C11=-2*m2*l1*lc2*dtheta2*sin(theta2);
C12=-m2*l1*lc2*dtheta2*sin(theta2);
C21=m2*l1*lc2*dtheta1*sin(theta2);
C22=0;
C=[C11,C12;C21,C22];

% (5)
% Vector de efectos de Gravedad
G11=(m1*g*lc1*cos(theta1))+(m2*g*l1*cos(theta1))+(m2*g*lc2*cos(theta1+theta2));
G12=m2*g*lc2*cos(theta1+theta2);
G=[G11;G12];

% (6)
% Vector de estados con las velocidades de los ngulos
dtheta=[dtheta1;dtheta2];

% (7)
% Vector de torques de entrada (en Pendubot, el segundo eslabn no tiene motor y el torque es siempre cero).
T=[u1;0];

% (8)
% Salida del modelo con la solucin inversa de variable de
% estado que se estudia en secciones posteriores.
y=-(D^(-1)*C*dtheta)-(D^(-1)*G)+(D^(-1)*T);
