% ACCEL Clculo de la Dinmica Directa
%
%   QDD = ACCEL(ROBOT, Q, QD, TORQUE)
%   QDD = ACCEL(ROBOT, [Q QD TORQUE])
%
% Resulta en el vector de aceleracin para cada
% articulacin del sistema, considerando el estado
% instantneo Q y la velocidad instantanea QD.
%
% Copyright (C) 1993-2008, by Peter I. Corke
% Robotics Toolbox for Matlab v8.0 

function qdd = accel(robot, Q, qd, torque)
n = robot.n;

% Clculo de la matriz de inercias D a partir de una
% aceleracin unitaria en cada articulacin.
% Considera una gravedad nula (ver ltimo argumento).

%NOTA (1)   ====================
M = rne(robot, ones(n,1)*q', zeros(n,n), eye(n), [0;0;0]);
 
% Calcula el torque de Coriolis y de la gravedad
% con aceleracin cero y la actuacin de la gravedad.

%NOTA (2)   ====================
tau = rne(robot, q', qd', zeros(1,n));  

% Solucin a la ecuacin de Walter y Orin
%NOTA (3)   ====================
qdd = inv(M) * (torque(:) - tau');
